import dotenv
import os
from celery import Celery

dotenv.load_dotenv()

celery_link = Celery('task',
                     broker=os.getenv("BROKER_URL"),
                     backend='db+postgresql+psycopg2://' + os.getenv("DATABASE_URL"))




from flask import Blueprint, request, redirect, render_template
import dotenv
from .creds import celery_link
import time
from .users_interactions import token_valid

dotenv.load_dotenv()

playlist_interaction = Blueprint('playlist_interaction', __name__, static_folder="../static",
                                 template_folder="../templates")


# Playlist CRUD

# Create
@playlist_interaction.route('/create', methods=['GET', 'POST'])
def create_playlist():
    if token_valid():
        if request.method == 'POST':
            name = request.form['playlist_name']
            token = request.cookies.get('session_token')
            celery_link.send_task("tasks.new_playlist", kwargs={'name': name, 'token': token})

            return redirect('/playlist')

        return render_template('create_playlist.html', title="Create Playlist")
    return redirect('/login')


# Read
def get_users_playlist():
    token = request.cookies.get('session_token')
    playlist_list = celery_link.send_task('tasks.get_user_playlists', kwargs={'token': token})
    while str(celery_link.AsyncResult(playlist_list.id).state) != "SUCCESS":
        time.sleep(0.25)
    playlist_list = celery_link.AsyncResult(playlist_list.id).result
    new_list = []
    for item in playlist_list:
        new_elm = {
            'id': item[0],
            'name': item[2]
        }
        new_list.append(new_elm)
    return new_list


@playlist_interaction.route('/')
def show_playlist():
    context = {
        'title': 'User Playlists'
    }

    # get playlist content
    return render_template('playlist.html', title=context['title'], playlists=get_users_playlist())


@playlist_interaction.route('/<int:id>')
def playlist_content(id):
    token = request.cookies.get('session_token')
    playlist_content = celery_link.send_task('tasks.show_playlist_content', kwargs={'playlist_id': id})
    while str(celery_link.AsyncResult(playlist_content.id).state) != "SUCCESS":
        time.sleep(0.25)
    playlist_content = celery_link.AsyncResult(playlist_content.id).result
    if len(playlist_content) == 0:
        redirect('/playlist')
    return render_template('playlist_content.html', playlist=playlist_content, title="User Playlist", playlist_id=id)


@playlist_interaction.route('/add', methods=['GET', 'POST'])
def playlist_add_song():
    token = request.cookies.get('session_token')
    if request.method == 'POST':
        playlist_id = request.form['playlist_id']
        add_song = celery_link.send_task('tasks.add_song_to_playlist',
                                         kwargs={'token': token, 'playlist_id': playlist_id,
                                                 'song_id': request.form['song_id']})
        while str(celery_link.AsyncResult(add_song.id).state) != "SUCCESS":
            time.sleep(0.25)
        add_song = celery_link.AsyncResult(add_song.id).result
        if add_song is False:
            return render_template('add_to_playlist.html', title="Add to Playlist", playlists=get_users_playlist(),
                                   message="Song ID not found")
        return redirect(f'/playlist/{playlist_id}')

    return render_template('add_to_playlist.html', title="Add to Playlist", playlists=get_users_playlist())


@playlist_interaction.route('/remove', methods=['POST'])
def playlist_remove_song():
    token = request.cookies.get('session_token')
    song_id = request.form['song_id']
    playlist_id = request.form['playlist_id']
    celery_link.send_task('tasks.remove_song_from_playlist',
                          kwargs={'song_id': song_id, 'playlist_id': playlist_id, 'token': token})

    return redirect(f'/playlist/{playlist_id}')


# Update
@playlist_interaction.route('/update/<int:id>', methods=['POST'])
def playlist_update_name(id):
    token = request.cookies.get('session_token')
    update_task = celery_link.send_task('tasks.update_playlist_name', kwargs={'token': token,
                                                                              'playlist_id': id,
                                                                              'new_name': request.form[
                                                                                  'playlist_name']})
    while str(celery_link.AsyncResult(update_task.id).state) != "SUCCESS":
        time.sleep(0.25)
    update_result = celery_link.AsyncResult(update_task.id).result
    return redirect('/playlist')


# Delete
@playlist_interaction.route('/delete/<int:id>')
def playlist_delete(id):
    celery_link.send_task('tasks.delete_playlist', kwargs={'token': request.cookies.get('session_token'),
                                                           'playlist_id': id})
    return redirect('/playlist')



import os
import time
import dotenv
from flask import Blueprint, render_template
from .creds import celery_link
from .crud import show_all_users

users = Blueprint("users", __name__, static_folder="../static", template_folder="../templates")

dotenv.load_dotenv()


@users.route('/show_users')
def show_users():
    title = "All Users"

    return render_template('user_list.html', title=title, users=show_all_users())




import os
import time
import dotenv
from flask import Blueprint, request, render_template, redirect, make_response
from .creds import celery_link

users_interactions = Blueprint("users_interactions", __name__, static_folder="../static",
                               template_folder="../templates")

dotenv.load_dotenv()


@users_interactions.route('/login', methods=["GET", "POST"])
def login():
    title = "login"
    if token_valid() is False:
        if request.method == 'GET':
            return render_template('login.html')
        if request.method == 'POST':
            username = request.form["username"]
            passwd = request.form["password"]
            login_request = celery_link.send_task("tasks.login", kwargs={'username': username, 'password': passwd})
            while str(celery_link.AsyncResult(login_request.id).state) != "SUCCESS":
                time.sleep(0.25)
            login_task_result = celery_link.AsyncResult(login_request.id).result
            if login_task_result is None:
                return render_template('login.html', title=title, message="ERROR: Credentials incorrect")
            if login_task_result[0]:
                if request.cookies.get('session_token') is None:
                    resp = make_response(redirect('/account'))
                    resp.set_cookie(key='session_token', value=login_task_result[1])
                    return resp
                return redirect('/account')
    return redirect('/account')


@users_interactions.route('/register', methods=["GET", "POST"])
def register():
    if not token_valid():
        if request.method == 'GET':
            return render_template('register.html')
        if request.method == 'POST':
            first = request.form["first"]
            last = request.form["last"]
            email = request.form["email"]
            usr = request.form["username"]
            password = request.form["password"]
            confirm = request.form["confirm"]
            register_tasks = celery_link.send_task("tasks.register",
                               kwargs={'first_name': first,
                                       'last_name': last,
                                       'email': email,
                                       'username': usr,
                                       'password': password})

            while str(celery_link.AsyncResult(register_tasks.id).state) != "SUCCESS":
                time.sleep(0.25)
            register_result = celery_link.AsyncResult(register_tasks.id).result
            if register_result:
                return redirect('/login')
            else:
                return render_template('register.html', message="Email already in use!", first=first, last=last, username=usr, password=password, confirm=confirm)
    return redirect('/account')


@users_interactions.route('/logout')
def logout():
    # Get session token
    session_token = request.cookies.get('session_token')
    # if token set
    if token_valid():
        logout_task = celery_link.send_task('tasks.logout', kwargs={'session_token': session_token})
        while str(celery_link.AsyncResult(logout_task.id).state) != "SUCCESS":
            time.sleep(0.25)
        register_result = celery_link.AsyncResult(logout_task.id).result
        if register_result:
            resp = make_response(redirect('/login'))
            resp.set_cookie('session_token', '', expires=0)
            return resp
        # Log out user
        # Redirect to home
    return redirect('/login')
    # redirect login


@users_interactions.route('/account')
def account_page():
    # Get session token
    session_token = request.cookies.get('session_token')
    if not token_valid():
        return redirect('/login')
    session_valid = celery_link.send_task('tasks.token_valid', kwargs={'session_token':session_token})
    while str(celery_link.AsyncResult(session_valid.id).state) != "SUCCESS":
        time.sleep(0.25)
    session_valid = celery_link.AsyncResult(session_valid.id).result
    # if token valid
    if session_valid:
        # Send token to backend asking for user information
        account_info = celery_link.send_task('tasks.user_info_from_session_token', kwargs={'session_token':session_token})
        while str(celery_link.AsyncResult(account_info.id).state) != "SUCCESS":
            time.sleep(0.25)
        # Get user info from backend
        account_info = celery_link.AsyncResult(account_info.id).result
        account_info = {
            "first_name": account_info[1],
            "last_name": account_info[2],
            "email": account_info[3],
            "username": account_info[4]
        }
        # Display user info on frontend
        return render_template('account_profile.html', account=account_info)
    return redirect('/login')


def token_valid():
    session_token = request.cookies.get('session_token')
    if session_token is not None:
        session_valid_task = celery_link.send_task('tasks.token_valid', kwargs={'session_token':session_token})
        while str(celery_link.AsyncResult(session_valid_task.id).state) != "SUCCESS":
            time.sleep(0.25)
        return celery_link.AsyncResult(session_valid_task.id).result
    return False




import os
import time
import dotenv
import requests
from flask import Blueprint, request, render_template
from .creds import celery_link

topten_charts = Blueprint("topcharts", __name__, static_folder="../static", template_folder="../templates")

dotenv.load_dotenv()


@topten_charts.route('/topcharts', methods=['GET', 'POST'])
def search():
    if request.method == "GET":
        context = {
            'title': 'topcharts'
        }
        return render_template('topcharts.html', title=context['title'])

    if request.method == "POST":
        title = 'topcharts'
        results = []
        genius_results = request_loadtopcharts_audiodb().json()

        for song in genius_results['loved']:
            # song = song['result']
            result = {
                'strAlbumThumb': song['strAlbumThumb'],
                'strAlbum': song['strAlbum'],
                'strArtist': song['strArtist'],
                'strGenre': song['strGenre'],
                'intYearReleased': song['intYearReleased']
            }
            # result['song_profile'] = f"/song?id={result['song_id']}"
            results.append(result)
            # results = genius_results

    return render_template('topcharts.html', title=title, results=results)

def request_loadtopcharts_audiodb():
    url = "https://theaudiodb.p.rapidapi.com/mostloved.php"

    querystring = {"format":"album"}

    headers = {
        'x-rapidapi-host': "theaudiodb.p.rapidapi.com",
        'x-rapidapi-key': "5bcf48bf11msh7e2498cfa2449c0p1b31fejsn11fb62150ba6"
    }

    response = requests.request("GET", url, headers=headers, params=querystring)
    print(response.text)
    return response


import os
import time
import dotenv
from flask import Blueprint, render_template
from .creds import celery_link

user_dump = Blueprint("user_dump", __name__, static_folder="../static", template_folder="../templates")

dotenv.load_dotenv()


@user_dump.route('/user_dump')
def users_show():
    title = "user dump"
    users = celery_link.send_task("tasks.get_latest_dump")
    while str(celery_link.AsyncResult(users.id).state) != "SUCCESS":
        time.sleep(0.1)
    users = celery_link.AsyncResult(users.id).result
    return render_template('latest_dump.html', title=title, users=users)



import os
import time
import dotenv
import requests
from flask import Blueprint, request, render_template
from .creds import celery_link

song_search = Blueprint("song_search", __name__, static_folder="../static", template_folder="../templates")

dotenv.load_dotenv()


@song_search.route('/search', methods=['GET', 'POST'])
def search():
    if request.method == "GET":
        context = {
            'title': 'Search'
        }
        return render_template('search.html', title=context['title'])

    if request.method == "POST":
        title = 'Search'
        results = []
        genius_results = request_song_info_genius(request.form['search_info']).json()
        for song in genius_results['response']['hits']:
            song = song['result']
            result = {
                'song_id': song['id'],
                'name': song['title'],
                'artist': song['artist_names'],
            }
            result['song_profile'] = f"/song?id={result['song_id']}"
            results.append(result)

        # return context['results']
        return render_template('search.html', title=title, results=results, placeholder=request.form['search_info'])


@song_search.route('/song', methods=['GET', 'POST'])
def song_profile():
    from urllib import parse

    title = "Song Profile Page"
    if request.args is not None:
        result = request_song_id_genius(request.args['id']).json()
        result = result['response']['song']
        result['liked'] = False
        for media in result['media']:
            if media['provider'] == "youtube":
                url = media['url']
                query_def = parse.parse_qs(parse.urlparse(url).query)['v'][0]
                result['youtube_id'] = query_def
        songInfo = {"title": result['title'], "artist": result["primary_artist"]["name"]}
        try:
            audioDBRes = request_song_info_audiodb(songInfo['artist'], songInfo['title']).json()
            result["genre"] = audioDBRes['track'][0]['strGenre']
        except TypeError:
            result["genre"] = "None"

        title = result['full_title']
        song_found = celery_link.send_task("tasks.find_song",
                                           kwargs={"name": songInfo['title'], "artist": songInfo['artist']})
        while str(celery_link.AsyncResult(song_found.id).state) != "SUCCESS":
            time.sleep(0.25)
        song_found_result = celery_link.AsyncResult(song_found.id).result
        if not song_found_result:
            celery_link.send_task("tasks.add_song", kwargs={"name": songInfo['title'], "artist": songInfo['artist'],
                                                            "genre": result['genre'], "genius_id": result['id']})
        else:
            song_liked = celery_link.send_task("tasks.get_liked_song", kwargs={"song_id": result['id'],
                                                                               "user_id": 1})  # TODO: Update user_id with dynamic user id
            while str(celery_link.AsyncResult(song_liked.id).state) != "SUCCESS":
                time.sleep(0.25)
            song_liked = celery_link.AsyncResult(song_liked.id).result
            result['liked'] = song_liked
        update_views = celery_link.send_task("tasks.update_views", kwargs={"genius_id": result['id']})
        while str(celery_link.AsyncResult(update_views.id).state) != "SUCCESS":
            time.sleep(0.25)
        count_views = celery_link.send_task("tasks.get_views", kwargs={"genius_id": result['id']})

        while str(celery_link.AsyncResult(count_views.id).state) != "SUCCESS":
            time.sleep(0.25)
        count_views = celery_link.AsyncResult(count_views.id).result
        result['views'] = count_views
        # to-do call celery link, call celery to increase views
    return render_template('song_profile.html', title=title, result=result)


# API CALLS

def request_song_info_genius(search_info):
    base_url = 'https://api.genius.com'
    headers = {'Authorization': 'Bearer ' + os.getenv('GENIUS_AUTH')}
    search_url = base_url + '/search'
    data = {'q': search_info}
    response = requests.get(search_url, data=data, headers=headers)

    return response


def request_song_id_genius(song_id):
    url = "https://genius-song-lyrics1.p.rapidapi.com/songs/" + song_id

    headers = {
        "X-RapidAPI-Host": "genius-song-lyrics1.p.rapidapi.com",
        "X-RapidAPI-Key": "e66edeed2emshe809355cbf14d21p15ef82jsnf921421de2fe"
    }

    response = requests.request("GET", url, headers=headers)

    return response


def request_song_info_spotify(search_info):
    url = "https://spotify23.p.rapidapi.com/search/"

    querystring = {"q": search_info, "type": "tracks", "limit": 15}

    headers = {
        'x-rapidapi-host': "spotify23.p.rapidapi.com",
        'x-rapidapi-key': "e66edeed2emshe809355cbf14d21p15ef82jsnf921421de2fe"
    }
    response = requests.get(url, headers=headers, params=querystring)

    return response

def request_loadtopcharts_audiodb():
    url = "https://theaudiodb.p.rapidapi.com/mostloved.php"

    querystring = {"format":"album"}

    headers = {
        'x-rapidapi-host': "theaudiodb.p.rapidapi.com",
        'x-rapidapi-key': "SIGN-UP-FOR-KEY"
    }

    response = requests.request("GET", url, headers=headers, params=querystring)

    return response

def request_song_info_audiodb(artist, track_name):
    url = "https://theaudiodb.p.rapidapi.com/searchtrack.php"

    querystring = {"s": artist, "t": track_name}

    headers = {
        'x-rapidapi-host': "theaudiodb.p.rapidapi.com",
        'x-rapidapi-key': "e66edeed2emshe809355cbf14d21p15ef82jsnf921421de2fe"
    }

    response = requests.get(url, headers=headers, params=querystring)

    return response


def request_song_by_id_spotify(song_id):
    url = "https://spotify23.p.rapidapi.com/tracks/"
    if type(song_id) == "string":
        querystring = {"ids": song_id}
    else:
        querystring = {"ids": ','.join(song_id)}

    headers = {
        'x-rapidapi-host': "spotify23.p.rapidapi.com",
        'x-rapidapi-key': "e66edeed2emshe809355cbf14d21p15ef82jsnf921421de2fe"
    }

    response = requests.get(url, headers=headers, params=querystring)

    return response



import os
import dotenv
from flask import Blueprint, render_template
from .creds import celery_link

dotenv.load_dotenv()

celery_test = Blueprint("celery_test", __name__, static_folder="../static", template_folder="../templates")


# Celery Test Code
@celery_test.route('/simple_start_task', methods=["POST"])
def call_method():
    print("Invoking Method ")
    r = celery_link.send_task('tasks.longtime_add', kwargs={'x': 1, 'y': 2})
    print(r.backend)
    return r.id


@celery_test.route('/simple_task_status/<task_id>')
def get_status(task_id):
    status = celery_link.AsyncResult(task_id, app=celery_link)
    print("Invoking Method ")
    return "Status of the Task " + str(status.state)


@celery_test.route('/simple_task_result/<task_id>')
def task_result(task_id):
    result = celery_link.AsyncResult(task_id).result
    return "Result of the Task " + str(result)

# End of Celery Test Code



import os
import time
import dotenv
import requests
from flask import Blueprint, request, render_template
from .creds import celery_link

top_songs = Blueprint("topsongs", __name__, static_folder="../static", template_folder="../templates")

dotenv.load_dotenv()


@top_songs.route('/topsongs', methods=['GET', 'POST'])
def search():
    if request.method == "GET":
        context = {
            'title': 'topsongs'
        }
        return render_template('topsongs.html', title=context['title'])

    if request.method == "POST":
        title = 'topsongs'
        results = []
        genius_results = request_loadtopsongs_audiodb().json()

        for song in genius_results['loved']:
            # song = song['result']
            if song['strGenre'] == request.form['genre'] or request.form['genre'] == "All":
                result = {
                    'strAlbum': song['strAlbum'],
                    'strArtist': song['strArtist'],
                    'strGenre': song['strGenre'],
                    'strTrack': song['strTrack'],
                    'intTotalPlays': song['intTotalPlays']
                }
                # result['song_profile'] = f"/song?id={result['song_id']}"
                results.append(result)
                # results = genius_results

    return render_template('topsongs.html', title=title, results=results)

def request_loadtopsongs_audiodb():
    url = "https://theaudiodb.p.rapidapi.com/mostloved.php"

    querystring = {"format":"track"}

    headers = {
        'x-rapidapi-host': "theaudiodb.p.rapidapi.com",
        'x-rapidapi-key': "5bcf48bf11msh7e2498cfa2449c0p1b31fejsn11fb62150ba6"
    }

    response = requests.request("GET", url, headers=headers, params=querystring)
    print(response.text)
    return response


from flask import Blueprint, request, redirect
import dotenv
from .creds import celery_link
import time

dotenv.load_dotenv()

song_interaction = Blueprint('song_interaction', __name__, static_folder="../static", template_folder="../templates")


def get_liked_song(genius_id, user_id):
    get_songs_task = celery_link.send_task("tasks.get_liked_song", kwargs={"song_id": genius_id, "user_id": user_id})
    while str(celery_link.AsyncResult(get_songs_task.id).state) != "SUCCESS":
        time.sleep(0.25)
    get_songs_result = celery_link.AsyncResult(get_songs_task.id).result
    return str(get_songs_result)


@song_interaction.route('/like_song')
def like_song():
    genius_id = int(request.args['genius_id'])
    user_id = int(request.args['user_id'])
    if get_liked_song(genius_id, user_id) == "False":
        task = celery_link.send_task("tasks.like_song", kwargs={"genius_id":genius_id, "user_id":user_id})
        while str(celery_link.AsyncResult(task.id).state) != "SUCCESS":
            time.sleep(0.25)
    return redirect(f'/song?id={genius_id}')


@song_interaction.route('/dislike_song')
def dislike_song():
    genius_id = int(request.args['genius_id'])
    user_id = int(request.args['user_id'])
    if get_liked_song(genius_id, user_id) == "True":
        task = celery_link.send_task("tasks.dislike_song", kwargs={"genius_id":genius_id, "user_id":user_id})
        while str(celery_link.AsyncResult(task.id).state) != "SUCCESS":
            time.sleep(0.25)
    return redirect(f'/song?id={genius_id}')


"""
  Crud Blueprint for flask
"""
import json
import os
import time
import dotenv
from flask import Blueprint, request
from .creds import celery_link
import requests

crud = Blueprint("crud", __name__, static_folder="../static", template_folder="../templates")

dotenv.load_dotenv()


def create_db():

    databasestatus = celery_link.send_task("tasks.create_db")
    while str(celery_link.AsyncResult(databasestatus.id).state) != "SUCCESS":
        time.sleep(0.1)

@crud.route("/add_test_news")
def add_test_news():
    news_task = celery_link.send_task("tasks.seed_if_empty")
    return news_task.id


@crud.route("/add_test_user")
def add_test_user():
    url = "https://my.api.mockaroo.com/it490___users.json"

    payload = {}
    headers = {
        'X-API-Key': '3f00bd30',
        'Content-Type': 'application/json',
        'Cookie': 'layer0_bucket=14; layer0_destination=default; layer0_environment_id_info=1680b086-a116-4dc7-a17d-9e6fdbb9f6d9'
    }

    response = requests.request("GET", url, headers=headers, data=payload)
    test_users = json.loads(response.text)

    user_task_ids = []
    for user in test_users:
        user_task = celery_link.send_task("tasks.add_user", kwargs={"first_name":user['first_name'],
                                                                    "last_name": user['last_name'],
                                                                    "email": user['email'],
                                                                    "username": user['username'],
                                                                    "password": user['password']})
        user_task_ids.append(user_task.id)
    return str(user_task_ids)


@crud.route("/add_user")
def add_user():
    user_task = celery_link.send_task("tasks.add_user", kwargs={"first_name": request.form["first_name"],
                                                                "last_name": request.form["last_name"],
                                                                "email": request.form["email"],
                                                                "username": request.form["username"],
                                                                "password": request.form["password"]})
    return user_task.id


@crud.route("/get_users")
def show_all_users():
    users_task = celery_link.send_task("tasks.get_users")
    while str(celery_link.AsyncResult(users_task.id).state) != "SUCCESS":
        time.sleep(0.25)
    users_result = celery_link.AsyncResult(users_task.id).result
    return str(users_result)


@crud.route("/get_user")
def show_user():
    users_task = celery_link.send_task("tasks.get_user", kwargs={"user_id": request.form['user_id']})
    while str(celery_link.AsyncResult(users_task.id).state) != "SUCCESS":
        time.sleep(0.25)
    users_result = celery_link.AsyncResult(users_task.id).result
    return users_result


@crud.route("/delete_user/<user_id>")
def delete_user(user_id):
    delete_task = celery_link.send_task("tasks.delete_user", kwargs={"user_id": user_id})
    while str(celery_link.AsyncResult(delete_task.id).state) != "SUCCESS":
        time.sleep(0.25)
    delete_task_result = celery_link.AsyncResult(delete_task.id).result
    return str(delete_task_result)


# Figure out later
@crud.route("/update_user/", methods=["GET", "POST"])
def update_user():
    if request.method == "GET":
        return "Sorry this is not implemented yet"
    if request.method == "POST":
        update_task = celery_link.send_task("tasks.update_user",
                                            kwargs={"id": request.form["id"], "first": request.form["first_name"],
                                                    "last": request.form["last_name"]})
        update_task_result = celery_link.AsyncResult(update_task.id).result
        return str(update_task_result)